@@ -1,5 +1,5 @@
 /* tree.c -- helper functions to build and evaluate the expression tree.
-   Copyright (C) 1990-2022 Free Software Foundation, Inc.
+   Copyright (C) 1990-2024 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,13 +23,11 @@
 #include <stdlib.h>
 
 /* gnulib headers. */
-#include "error.h"
-#include "fnmatch.h"
+#include <fnmatch.h>
 #include "xalloc.h"
 
 /* find headers. */
 #include "defs.h"
-#include "die.h"
 #include "system.h"
 
 
@@ -112,28 +110,28 @@ get_expr (struct predicate **input,
   struct predicate *this_pred = (*input);
 
   if (*input == NULL)
-    die (EXIT_FAILURE, 0, _("invalid expression"));
+    error (EXIT_FAILURE, 0, _("invalid expression"));
 
   switch ((*input)->p_type)
     {
     case NO_TYPE:
-      die (EXIT_FAILURE, 0, _("invalid expression"));
+      error (EXIT_FAILURE, 0, _("invalid expression"));
       break;
 
     case BI_OP:
       /* e.g. "find . -a" */
-      die (EXIT_FAILURE, 0,
-	   _("invalid expression; you have used a binary operator '%s' with nothing before it."),
-	   this_pred->p_name);
+      error (EXIT_FAILURE, 0,
+	     _("invalid expression; you have used a binary operator '%s' with nothing before it."),
+	     this_pred->p_name);
       break;
 
     case CLOSE_PAREN:
       if (prev_pred == NULL)
 	{
 	  /* Happens with e.g. "find -files0-from - ')' -print" */
-	  die (EXIT_FAILURE, 0,
-	       _("invalid expression: expected expression before closing parentheses '%s'."),
-	       this_pred->p_name);
+	  error (EXIT_FAILURE, 0,
+		 _("invalid expression: expected expression before closing parentheses '%s'."),
+		 this_pred->p_name);
 	}
 
       if ((UNI_OP == prev_pred->p_type
@@ -141,9 +139,9 @@ get_expr (struct predicate **input,
 	  && !this_pred->artificial)
 	{
 	  /* e.g. "find \( -not \)" or "find \( -true -a \)" */
-	  die (EXIT_FAILURE, 0,
-	       _("expected an expression between '%s' and ')'"),
-	       prev_pred->p_name);
+	  error (EXIT_FAILURE, 0,
+		 _("expected an expression between '%s' and ')'"),
+		 prev_pred->p_name);
 	}
       else if ( (*input)->artificial )
 	{
@@ -151,13 +149,13 @@ get_expr (struct predicate **input,
 	   * unexpectedly.
 	   */
 	  /* e.g. "find . -true -a" */
-	  die (EXIT_FAILURE, 0,
-	       _("expected an expression after '%s'"), prev_pred->p_name);
+	  error (EXIT_FAILURE, 0,
+		 _("expected an expression after '%s'"), prev_pred->p_name);
 	}
       else
 	{
-	  die (EXIT_FAILURE, 0,
-	       _("invalid expression; you have too many ')'"));
+	  error (EXIT_FAILURE, 0,
+		 _("invalid expression; you have too many ')'"));
 	}
       break;
 
@@ -179,10 +177,10 @@ get_expr (struct predicate **input,
 	   * looking at is from the artificial "( ) -print" that we
 	   * add.
 	   */
-	  die (EXIT_FAILURE, 0,
-	       _("invalid expression; expected to find a ')' but didn't see one. "
-		 "Perhaps you need an extra predicate after '%s'"),
-	       this_pred->p_name);
+	  error (EXIT_FAILURE, 0,
+		 _("invalid expression; expected to find a ')' but didn't see one. "
+		   "Perhaps you need an extra predicate after '%s'"),
+		 this_pred->p_name);
 	}
       prev_pred = (*input);
       *input = (*input)->pred_next;
@@ -190,25 +188,25 @@ get_expr (struct predicate **input,
 	{
 	  if (prev_pred->artificial)
 	    {
-	      die (EXIT_FAILURE, 0,
-		   _("invalid expression: expected expression before closing parentheses '%s'."),
-		   (*input)->p_name);
+	      error (EXIT_FAILURE, 0,
+		     _("invalid expression: expected expression before closing parentheses '%s'."),
+		     (*input)->p_name);
 	    }
-	  die (EXIT_FAILURE, 0,
-	       _("invalid expression; empty parentheses are not allowed."));
+	  error (EXIT_FAILURE, 0,
+		 _("invalid expression; empty parentheses are not allowed."));
 	}
       next = get_expr (input, NO_PREC, prev_pred);
       if ((*input == NULL)
 	  || ((*input)->p_type != CLOSE_PAREN))
-	die (EXIT_FAILURE, 0,
-	     _("invalid expression; I was expecting to find a ')' somewhere "
-	       "but did not see one."));
+	error (EXIT_FAILURE, 0,
+	       _("invalid expression; I was expecting to find a ')' somewhere "
+		 "but did not see one."));
 
       *input = (*input)->pred_next;	/* move over close */
       break;
 
     default:
-      die (EXIT_FAILURE, 0, _("oops -- invalid expression type!"));
+      error (EXIT_FAILURE, 0, _("oops -- invalid expression type!"));
       break;
     }
 
@@ -223,7 +221,7 @@ get_expr (struct predicate **input,
     {
       next = scan_rest (input, next, prev_prec);
       if (next == NULL)
-	die (EXIT_FAILURE, 0, _("invalid expression"));
+	error (EXIT_FAILURE, 0, _("invalid expression"));
     }
   return (next);
 }
@@ -262,7 +260,7 @@ scan_rest (struct predicate **input,
 	  /* I'm not sure how we get here, so it is not obvious what
 	   * sort of mistakes might give rise to this condition.
 	   */
-	  die (EXIT_FAILURE, 0, _("invalid expression"));
+	  error (EXIT_FAILURE, 0, _("invalid expression"));
 	  break;
 
 	case BI_OP:
@@ -279,9 +277,9 @@ scan_rest (struct predicate **input,
 	  return tree;
 
 	default:
-	  die (EXIT_FAILURE, 0,
-	       _("oops -- invalid expression type (%d)!"),
-	       (int)(*input)->p_type);
+	  error (EXIT_FAILURE, 0,
+		 _("oops -- invalid expression type (%d)!"),
+		 (int)(*input)->p_type);
 	  break;
 	}
     }
@@ -298,7 +296,7 @@ predicate_is_cost_free (const struct predicate *p)
       pred_is(p, pred_ipath))
     {
       /* Traditionally (at least 4.1.7 through 4.2.x) GNU find always
-       * optimised these cases.
+       * optimized these cases.
        */
       return true;
     }
@@ -531,7 +529,7 @@ consider_arm_swap (struct predicate *p)
 {
   int left_cost, right_cost;
   const char *reason = NULL;
-  struct predicate **pl, **pr;
+  struct predicate **pl = NULL, **pr = NULL;
 
   if (BI_OP != p->p_type)
     reason = "Not a binary operation";
@@ -542,14 +540,18 @@ consider_arm_swap (struct predicate *p)
 	reason = "Doesn't have two arms";
     }
 
-
   if (!reason)
     {
       if (NULL == p->pred_left->pred_right)
-	reason = "Left arm has no child on RHS";
+	{
+	  reason = "Left arm has no child on RHS";
+	}
+      else
+	{
+	  pr = &p->pred_right;
+	  pl = &p->pred_left->pred_right;
+	}
     }
-  pr = &p->pred_right;
-  pl = &p->pred_left->pred_right;
 
   if (!reason)
     {
@@ -832,7 +834,7 @@ opt_expr (struct predicate **eval_treep)
 	     all of the user's parentheses. */
 
 	default:
-	  die (EXIT_FAILURE, 0, _("oops -- invalid expression type!"));
+	  error (EXIT_FAILURE, 0, _("oops -- invalid expression type!"));
 	  break;
 	}
 
@@ -1058,7 +1060,7 @@ get_pred_cost (const struct predicate *p)
       data_requirement_cost = NeedsNothing;
     }
 
-  if (pred_is (p, pred_exec) || pred_is(p, pred_execdir))
+  if (predicate_uses_exec (p))
     {
       if (p->args.exec_vec.multiple)
 	inherent_cost = NeedsEventualExec;
@@ -1087,7 +1089,7 @@ get_pred_cost (const struct predicate *p)
 			     sizeof(costlookup[0]),
 			     cost_table_comparison))
 	    {
-	      die (EXIT_FAILURE, 0, "failed to sort the costlookup array");
+	      error (EXIT_FAILURE, 0, _("failed to sort the costlookup array"));
 	    }
 	  pred_table_sorted = 1;
 	}
@@ -1295,7 +1297,7 @@ build_expression_tree (int argc, char *argv[], int end_of_leading_options)
       if (parse_entry == NULL)
 	{
 	  /* Command line option not recognized */
-	  die (EXIT_FAILURE, 0, _("unknown predicate `%s'"), predicate_name);
+	  error (EXIT_FAILURE, 0, _("unknown predicate `%s'"), predicate_name);
 	}
 
       /* We have recognised a test of the form -foo.  Eat that,
@@ -1315,18 +1317,18 @@ build_expression_tree (int argc, char *argv[], int end_of_leading_options)
 		  /* The special parse function spat out the
 		   * predicate.  It must be invalid, or not tasty.
 		   */
-		  die (EXIT_FAILURE, 0, _("invalid predicate `%s'"), predicate_name);
+		  error (EXIT_FAILURE, 0, _("invalid predicate `%s'"), predicate_name);
 		}
 	      else
 		{
-		  die (EXIT_FAILURE, 0, _("invalid argument `%s' to `%s'"),
-		       argv[i], predicate_name);
+		  error (EXIT_FAILURE, 0, _("invalid argument `%s' to `%s'"),
+		         argv[i], predicate_name);
 		}
 	    }
 	  else
 	    {
 	      /* Command line option requires an argument */
-	      die (EXIT_FAILURE, 0, _("missing argument to `%s'"), predicate_name);
+	      error (EXIT_FAILURE, 0, _("missing argument to `%s'"), predicate_name);
 	    }
 	}
       else
@@ -1402,15 +1404,15 @@ build_expression_tree (int argc, char *argv[], int end_of_leading_options)
       if (pred_is (cur_pred, pred_closeparen))
 	{
 	  /* e.g. "find \( -true \) \)" */
-	  die (EXIT_FAILURE, 0, _("you have too many ')'"));
+	  error (EXIT_FAILURE, 0, _("you have too many ')'"));
 	}
       else
 	{
 	  if (cur_pred->p_name)
-	    die (EXIT_FAILURE, 0,
-		 _("unexpected extra predicate '%s'"), cur_pred->p_name);
+	    error (EXIT_FAILURE, 0,
+		   _("unexpected extra predicate '%s'"), cur_pred->p_name);
 	  else
-	    die (EXIT_FAILURE, 0, _("unexpected extra predicate"));
+	    error (EXIT_FAILURE, 0, _("unexpected extra predicate"));
 	}
     }
 
@@ -1428,7 +1430,10 @@ build_expression_tree (int argc, char *argv[], int end_of_leading_options)
   /* Check that the tree is in normalised order (opt_expr does this) */
   check_normalization (eval_tree, true);
 
-  do_arm_swaps (eval_tree);
+  if (options.optimisation_level > 1)
+    {
+      do_arm_swaps (eval_tree);
+    }
 
   /* Check that the tree is still in normalised order */
   check_normalization (eval_tree, true);
@@ -1503,7 +1508,6 @@ get_new_pred (const struct parser_table *entry)
   last_pred->need_type = true;
   last_pred->p_cost = NeedsUnknown;
   last_pred->arg_text = "ThisShouldBeSetToSomethingElse";
-  last_pred->literal_control_chars = options.literal_control_chars;
   last_pred->est_success_rate = 1.0;
   init_pred_perf (last_pred);
   return last_pred;
@@ -1531,7 +1535,7 @@ get_new_pred_chk_op (const struct parser_table *entry,
     switch (last_pred->p_type)
       {
       case NO_TYPE:
-	die (EXIT_FAILURE, 0, _("oops -- invalid default insertion of and!"));
+	error (EXIT_FAILURE, 0, _("oops -- invalid default insertion of and!"));
 	break;
 
       case PRIMARY_TYPE:
