@@ -1,5 +1,5 @@
 /* A simple (device, inode) struct.
-   Copyright (C) 2003-2022 Free Software Foundation, Inc.
+   Copyright (C) 2003-2024 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,10 +17,15 @@
 /* Written by Jim Meyering, 2003.  */
 
 #ifndef DEV_INO_H
-# define DEV_INO_H 1
+#define DEV_INO_H 1
+
+#include <sys/types.h>
+#include <sys/stat.h>
+
+#ifdef __cplusplus
+extern "C" {
+#endif
 
-# include <sys/types.h>
-# include <sys/stat.h>
 
 struct dev_ino
 {
@@ -28,4 +33,9 @@ struct dev_ino
   dev_t st_dev;
 };
 
+
+#ifdef __cplusplus
+}
+#endif
+
 #endif
