@@ -1,11 +1,11 @@
 /* Declare __fpending.
 
-   Copyright (C) 2000, 2003, 2005-2006, 2009-2019 Free Software Foundation,
+   Copyright (C) 2000, 2003, 2005-2006, 2009-2024 Free Software Foundation,
    Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3 of the License, or
+   the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -18,12 +18,28 @@
 
    Written by Jim Meyering.  */
 
+/* This file uses _GL_ATTRIBUTE_PURE, HAVE_STDIO_EXT_H,
+   HAVE_DECL___FPENDING.  */
+#if !_GL_CONFIG_H_INCLUDED
+ #error "Please include config.h first."
+#endif
+
 #include <stddef.h>
 #include <stdio.h>
 #if HAVE_STDIO_EXT_H
 # include <stdio_ext.h>
 #endif
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+
 #if !HAVE_DECL___FPENDING
 size_t __fpending (FILE *) _GL_ATTRIBUTE_PURE;
 #endif
+
+
+#ifdef __cplusplus
+}
+#endif
