@@ -1,6 +1,6 @@
 /* A more useful interface to strtol.
 
-   Copyright (C) 1995-1996, 1998-1999, 2001-2004, 2006-2022 Free Software
+   Copyright (C) 1995-1996, 1998-1999, 2001-2004, 2006-2024 Free Software
    Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
@@ -17,12 +17,17 @@
    along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 #ifndef XSTRTOL_H_
-# define XSTRTOL_H_ 1
+#define XSTRTOL_H_ 1
 
 /* Get intmax_t, uintmax_t.  */
-# include <stdint.h>
+#include <stdint.h>
 
-# ifndef _STRTOL_ERROR
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+
+#ifndef _STRTOL_ERROR
 enum strtol_error
   {
     LONGINT_OK = 0,
@@ -37,9 +42,9 @@ enum strtol_error
     LONGINT_INVALID = 4
   };
 typedef enum strtol_error strtol_error;
-# endif
+#endif
 
-# define _DECLARE_XSTRTOL(name, type) \
+#define _DECLARE_XSTRTOL(name, type) \
   strtol_error name (const char *, char **, int, type *, const char *);
 _DECLARE_XSTRTOL (xstrtol, long int)
 _DECLARE_XSTRTOL (xstrtoul, unsigned long int)
@@ -48,4 +53,9 @@ _DECLARE_XSTRTOL (xstrtoull, unsigned long long int)
 _DECLARE_XSTRTOL (xstrtoimax, intmax_t)
 _DECLARE_XSTRTOL (xstrtoumax, uintmax_t)
 
+
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* not XSTRTOL_H_ */
