@@ -1,5 +1,5 @@
 /* Test changing to a directory.
-   Copyright (C) 2011-2022 Free Software Foundation, Inc.
+   Copyright (C) 2011-2024 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -29,5 +29,5 @@ main (void)
 {
   ASSERT (chdir ("/") == 0);
 
-  return 0;
+  return test_exit_status;
 }
