@@ -1,5 +1,5 @@
 /* Test the simple ring buffer.
-   Copyright (C) 2006-2022 Free Software Foundation, Inc.
+   Copyright (C) 2006-2024 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -59,5 +59,5 @@ main (void)
   ASSERT (o == 8);
   ASSERT (i_ring_empty (&ir));
 
-  return 0;
+  return test_exit_status;
 }
