@@ -1,5 +1,5 @@
 /* Test the read() function.
-   Copyright (C) 2011-2022 Free Software Foundation, Inc.
+   Copyright (C) 2011-2024 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -69,5 +69,5 @@ main (void)
   /* Clean up.  */
   unlink (filename);
 
-  return 0;
+  return test_exit_status;
 }
