@@ -1,5 +1,5 @@
 /* Test of rounding towards zero.
-   Copyright (C) 2007-2022 Free Software Foundation, Inc.
+   Copyright (C) 2007-2024 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -24,7 +24,6 @@
 #include <math.h>
 
 #include <float.h>
-#include <stdbool.h>
 #include <stdint.h>
 #include <stdio.h>
 
@@ -174,5 +173,5 @@ main ()
                      >> NUM_HIGHBITS);
         error |= check (janus.f);
       }
-  return (error ? 1 : 0);
+  return (error ? 1 : test_exit_status);
 }
