@@ -1,11 +1,11 @@
 #!/bin/sh
 # Test suite for update-copyright.
-# Copyright (C) 2009-2019 Free Software Foundation, Inc.
+# Copyright (C) 2009-2024 Free Software Foundation, Inc.
 # This file is part of the GNUlib Library.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 3 of the License, or
+# the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
@@ -28,7 +28,7 @@ PATH=$abs_aux_dir:$PATH
 export PATH
 
 TMP_BASE=update-copyright.test
-trap 'rm -f $TMP_BASE*' 0 1 2 3 15
+trap 'rm -f $TMP_BASE*' EXIT HUP INT QUIT TERM
 
 ## --------------------------------- ##
 ## Skip if user does not have perl.  ##
@@ -37,9 +37,23 @@ trap 'rm -f $TMP_BASE*' 0 1 2 3 15
 TMP=$TMP_BASE
 s=$TMP-script
 cat <<\EOF > $s
-eval '(exit $?0)' && eval 'exec perl -wS -0777 -pi "$0" "$@"'
-  & eval 'exec perl -wS -0777 -pi "$0" $argv:q'
-    if 0;
+#!/bin/sh
+#! -*-perl-*-
+# This is a prologue that allows to run a perl script as an executable
+# on systems that are compliant to a POSIX version before POSIX:2017.
+# On such systems, the usual invocation of an executable through execlp()
+# or execvp() fails with ENOEXEC if it is a script that does not start
+# with a #! line.  The script interpreter mentioned in the #! line has
+# to be /bin/sh, because on GuixSD systems that is the only program that
+# has a fixed file name.  The second line is essential for perl and is
+# also useful for editing this file in Emacs.  The next two lines below
+# are valid code in both sh and perl.  When executed by sh, they re-execute
+# the script through the perl program found in $PATH.  The '-x' option
+# is essential as well; without it, perl would re-execute the script
+# through /bin/sh.  When executed by perl, the next two lines are a no-op.
+eval 'exec perl -wSx -pi "$0" "$@"'
+     if 0;
+
 s/a/b/
 EOF
 chmod a+x $s
@@ -54,7 +68,7 @@ echo a > $TMP-in
 # Skip this test if Perl is too old.  FIXME: 5.8.0 is just a guess.
 # We have a report that 5.6.1 is inadequate and that 5.8.0 works.
 perl -e 'require 5.8.0' || {
-  echo '$0: skipping this test; Perl version is too old' 1>&2
+  echo "$0: skipping this test; Perl version is too old" 1>&2
   exit 77
 }
 
@@ -62,170 +76,241 @@ perl -e 'require 5.8.0' || {
 UPDATE_COPYRIGHT_MAX_LINE_LENGTH=72
 export UPDATE_COPYRIGHT_MAX_LINE_LENGTH
 
+# This assignment to 'ight' is so that data can contain strings
+# like "Copy$ight 2024 Free Software Foundation, Inc." without being
+# mistakenly updated by "cd .. && make update-copyright".
+ight=right
+
 ## ----------------------------- ##
 ## Examples from documentation.  ##
 ## ----------------------------- ##
 
 TMP=$TMP_BASE-ex
-cat > $TMP.1 <<EOF
-Copyright @copyright{} 1990-2005, 2007-2009 Free Software
+cat > $TMP-1 <<EOF
+Copy$ight @copy$ight{} 1990-2005, 2007-2009 Free Software
 Foundation, Inc.
 EOF
-cat > $TMP.2 <<EOF
-# Copyright (C) 1990-2005, 2007-2009 Free Software
+cat > $TMP-2 <<EOF
+# Copy$ight (C) 1990-2005, 2007-2009 Free Software
 # Foundation, Inc.
 EOF
-cat > $TMP.3 <<EOF
+cat > $TMP-3 <<EOF
 /*
- * Copyright &copy; 90,2005,2007-2009
+ * Copy$ight &copy; 90,2005,2007-2009
  * Free Software Foundation, Inc.
  */
 EOF
-cat > $TMP.4 <<EOF
-## Copyright (C) 1990-2005, 2007-2009 Free Software
+cat > $TMP-4 <<EOF
+## Copy$ight (C) 1990-2005, 2007-2009 Free Software
 #  Foundation, Inc.
 EOF
-cat > $TMP.5 <<EOF
-Copyright (C) 1990-2005, 2007-2009 Acme, Inc.
+cat > $TMP-5 <<EOF
+Copy$ight (C) 1990-2005, 2007-2009 Acme, Inc.
 EOF
-cat > $TMP.6 <<EOF
-## Copyright (C) 1990-2005, 2007-2009 Free Software
+cat > $TMP-6 <<EOF
+## Copy$ight (C) 1990-2005, 2007-2009 Free Software
 #  Foundation, Inc.
 
-Copyright (C) 1990-2005, 2007-2009 Free Software Foundation,
+Copy$ight (C) 1990-2005, 2007-2009 Free Software Foundation,
 Inc.
 EOF
-cat > $TMP.7 <<EOF
-Copyright (C) 1990-2005, 2007-2009 Acme, Inc.
+cat > $TMP-7 <<EOF
+Copy$ight (C) 1990-2005, 2007-2009 Acme, Inc.
 
-# Copyright (C) 1990-2005, 2007-2009 Free Software
+# Copy$ight (C) 1990-2005, 2007-2009 Free Software
 # Foundation, Inc.
 EOF
+cat > $TMP-8 <<EOF
+Copy$ight (C) 2008 Free Software Foundation, Inc.
+Copy$ight (C) 2008 Free Software Foundation, Inc.
+EOF
+cat > $TMP-9 <<EOF
+# Copy$ight (C) 2008 Free Software Foundation, Inc.
+
+Copy$ight (C) 2007 Free Software Foundation, Inc.
+
+Copy$ight (C) 2008 Free Software Foundation, Inc.
+EOF
 
 UPDATE_COPYRIGHT_YEAR=2009 \
-  update-copyright $TMP.? 1> $TMP-stdout 2> $TMP-stderr
+  update-copyright $TMP-? 1> $TMP-stdout 2> $TMP-stderr
 compare /dev/null $TMP-stdout || exit 1
 compare - $TMP-stderr <<EOF || exit 1
-$TMP.4: warning: copyright statement not found
-$TMP.5: warning: copyright statement not found
+$TMP-4: warning: copyright statement not found
+$TMP-5: warning: copyright statement not found
 EOF
-compare - $TMP.1 <<EOF || exit 1
-Copyright @copyright{} 1990-2005, 2007-2009 Free Software
+compare - $TMP-1 <<EOF || exit 1
+Copy$ight @copy$ight{} 1990-2005, 2007-2009 Free Software
 Foundation, Inc.
 EOF
-compare - $TMP.2 <<EOF || exit 1
-# Copyright (C) 1990-2005, 2007-2009 Free Software
+compare - $TMP-2 <<EOF || exit 1
+# Copy$ight (C) 1990-2005, 2007-2009 Free Software
 # Foundation, Inc.
 EOF
-compare - $TMP.3 <<EOF || exit 1
+compare - $TMP-3 <<EOF || exit 1
 /*
- * Copyright &copy; 90,2005,2007-2009
+ * Copy$ight &copy; 90,2005,2007-2009
  * Free Software Foundation, Inc.
  */
 EOF
-compare - $TMP.4 <<EOF || exit 1
-## Copyright (C) 1990-2005, 2007-2009 Free Software
+compare - $TMP-4 <<EOF || exit 1
+## Copy$ight (C) 1990-2005, 2007-2009 Free Software
 #  Foundation, Inc.
 EOF
-compare - $TMP.5 <<EOF || exit 1
-Copyright (C) 1990-2005, 2007-2009 Acme, Inc.
+compare - $TMP-5 <<EOF || exit 1
+Copy$ight (C) 1990-2005, 2007-2009 Acme, Inc.
 EOF
-compare - $TMP.6 <<EOF || exit 1
-## Copyright (C) 1990-2005, 2007-2009 Free Software
+compare - $TMP-6 <<EOF || exit 1
+## Copy$ight (C) 1990-2005, 2007-2009 Free Software
 #  Foundation, Inc.
 
-Copyright (C) 1990-2005, 2007-2009 Free Software Foundation,
+Copy$ight (C) 1990-2005, 2007-2009 Free Software Foundation,
 Inc.
 EOF
-compare - $TMP.7 <<EOF || exit 1
-Copyright (C) 1990-2005, 2007-2009 Acme, Inc.
+compare - $TMP-7 <<EOF || exit 1
+Copy$ight (C) 1990-2005, 2007-2009 Acme, Inc.
 
-# Copyright (C) 1990-2005, 2007-2009 Free Software
+# Copy$ight (C) 1990-2005, 2007-2009 Free Software
 # Foundation, Inc.
 EOF
+compare - $TMP-8 <<EOF || exit 1
+Copy$ight (C) 2008, 2009 Free Software Foundation, Inc.
+Copy$ight (C) 2008, 2009 Free Software Foundation, Inc.
+EOF
+
+# Note that expecting "2007, 2009" on the third copyright line below
+# looks wrong since the corresponding copyright year was "2008".
+# However, the premise of this test (having inconsistent copyright dates
+# with the same holder) is not worth handling. What happens is that the
+# xform for the preceding line matches any list of year numbers with the
+# same prefix and holder.
+compare - $TMP-9 <<EOF || exit 1
+# Copy$ight (C) 2008, 2009 Free Software Foundation, Inc.
+
+Copy$ight (C) 2007, 2009 Free Software Foundation, Inc.
+
+Copy$ight (C) 2007, 2009 Free Software Foundation, Inc.
+EOF
+
+UPDATE_COPYRIGHT_YEAR=2010 UPDATE_COPYRIGHT_USE_INTERVALS=1 \
+  update-copyright $TMP-? 1> $TMP-stdout 2> $TMP-stderr
+compare /dev/null $TMP-stdout || exit 1
+compare - $TMP-stderr <<EOF || exit 1
+$TMP-4: warning: copyright statement not found
+$TMP-5: warning: copyright statement not found
+EOF
+compare - $TMP-1 <<EOF || exit 1
+Copy$ight @copy$ight{} 1990-2005, 2007-2010 Free Software Foundation,
+Inc.
+EOF
+compare - $TMP-2 <<EOF || exit 1
+# Copy$ight (C) 1990-2005, 2007-2010 Free Software Foundation, Inc.
+EOF
+compare - $TMP-3 <<EOF || exit 1
+/*
+ * Copy$ight &copy; 1990, 2005, 2007-2010 Free Software Foundation, Inc.
+ */
+EOF
+compare - $TMP-4 <<EOF || exit 1
+## Copy$ight (C) 1990-2005, 2007-2009 Free Software
+#  Foundation, Inc.
+EOF
+compare - $TMP-5 <<EOF || exit 1
+Copy$ight (C) 1990-2005, 2007-2009 Acme, Inc.
+EOF
+compare - $TMP-6 <<EOF || exit 1
+## Copy$ight (C) 1990-2005, 2007-2009 Free Software
+#  Foundation, Inc.
+
+Copy$ight (C) 1990-2005, 2007-2010 Free Software Foundation, Inc.
+EOF
+compare - $TMP-7 <<EOF || exit 1
+Copy$ight (C) 1990-2005, 2007-2009 Acme, Inc.
+
+# Copy$ight (C) 1990-2005, 2007-2010 Free Software Foundation, Inc.
+EOF
+
+UPDATE_COPYRIGHT_YEAR=2010 UPDATE_COPYRIGHT_FORCE=1 \
+  update-copyright $TMP-? 1> $TMP-stdout 2> $TMP-stderr
+compare /dev/null $TMP-stdout || exit 1
+compare - $TMP-stderr <<EOF || exit 1
+$TMP-4: warning: copyright statement not found
+$TMP-5: warning: copyright statement not found
+EOF
+compare - $TMP-1 <<EOF || exit 1
+Copy$ight @copy$ight{} 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997,
+1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2007, 2008, 2009, 2010
+Free Software Foundation, Inc.
+EOF
+compare - $TMP-2 <<EOF || exit 1
+# Copy$ight (C) 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998,
+# 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2007, 2008, 2009, 2010 Free
+# Software Foundation, Inc.
+EOF
+compare - $TMP-3 <<EOF || exit 1
+/*
+ * Copy$ight &copy; 1990, 2005, 2007, 2008, 2009, 2010 Free Software
+ * Foundation, Inc.
+ */
+EOF
+compare - $TMP-4 <<EOF || exit 1
+## Copy$ight (C) 1990-2005, 2007-2009 Free Software
+#  Foundation, Inc.
+EOF
+compare - $TMP-5 <<EOF || exit 1
+Copy$ight (C) 1990-2005, 2007-2009 Acme, Inc.
+EOF
+compare - $TMP-6 <<EOF || exit 1
+## Copy$ight (C) 1990-2005, 2007-2009 Free Software
+#  Foundation, Inc.
+
+Copy$ight (C) 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998,
+1999, 2000, 2001, 2002, 2003, 2004, 2005, 2007, 2008, 2009, 2010 Free
+Software Foundation, Inc.
+EOF
+compare - $TMP-7 <<EOF || exit 1
+Copy$ight (C) 1990-2005, 2007-2009 Acme, Inc.
+
+# Copy$ight (C) 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998,
+# 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2007, 2008, 2009, 2010 Free
+# Software Foundation, Inc.
+EOF
+
+rm $TMP*
+
+## ------------------------ ##
+## Examples from man pages  ##
+## ------------------------ ##
+
+TMP=$TMP_BASE-man
+cat > $TMP.1 <<EOF
+'\" Copy$ight (C) 1998-2009 Free Software Foundation, Inc.
+EOF
+cat > $TMP.2 <<EOF
+Copy$ight \(co 1998-2009 Free Software Foundation, Inc.
+EOF
+cat > $TMP.3 <<EOF
+Copy$ight \(co 1998\(en2009 Free Software Foundation, Inc.
+EOF
+cat > $TMP.4 <<EOF
+Copy$ight \(co 2009 Free Software Foundation, Inc.
+EOF
 
 UPDATE_COPYRIGHT_YEAR=2010 UPDATE_COPYRIGHT_USE_INTERVALS=1 \
   update-copyright $TMP.? 1> $TMP-stdout 2> $TMP-stderr
 compare /dev/null $TMP-stdout || exit 1
-compare - $TMP-stderr <<EOF || exit 1
-$TMP.4: warning: copyright statement not found
-$TMP.5: warning: copyright statement not found
-EOF
+compare /dev/null $TMP-stderr || exit 1
 compare - $TMP.1 <<EOF || exit 1
-Copyright @copyright{} 1990-2005, 2007-2010 Free Software Foundation,
-Inc.
+'\" Copy$ight (C) 1998\(en2010 Free Software Foundation, Inc.
 EOF
 compare - $TMP.2 <<EOF || exit 1
-# Copyright (C) 1990-2005, 2007-2010 Free Software Foundation, Inc.
+Copy$ight \(co 1998\(en2010 Free Software Foundation, Inc.
 EOF
 compare - $TMP.3 <<EOF || exit 1
-/*
- * Copyright &copy; 1990, 2005, 2007-2010 Free Software Foundation, Inc.
- */
+Copy$ight \(co 1998\(en2010 Free Software Foundation, Inc.
 EOF
 compare - $TMP.4 <<EOF || exit 1
-## Copyright (C) 1990-2005, 2007-2009 Free Software
-#  Foundation, Inc.
-EOF
-compare - $TMP.5 <<EOF || exit 1
-Copyright (C) 1990-2005, 2007-2009 Acme, Inc.
-EOF
-compare - $TMP.6 <<EOF || exit 1
-## Copyright (C) 1990-2005, 2007-2009 Free Software
-#  Foundation, Inc.
-
-Copyright (C) 1990-2005, 2007-2010 Free Software Foundation, Inc.
-EOF
-compare - $TMP.7 <<EOF || exit 1
-Copyright (C) 1990-2005, 2007-2009 Acme, Inc.
-
-# Copyright (C) 1990-2005, 2007-2010 Free Software Foundation, Inc.
-EOF
-
-UPDATE_COPYRIGHT_YEAR=2010 UPDATE_COPYRIGHT_FORCE=1 \
-  update-copyright $TMP.? 1> $TMP-stdout 2> $TMP-stderr
-compare /dev/null $TMP-stdout || exit 1
-compare - $TMP-stderr <<EOF || exit 1
-$TMP.4: warning: copyright statement not found
-$TMP.5: warning: copyright statement not found
-EOF
-compare - $TMP.1 <<EOF || exit 1
-Copyright @copyright{} 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997,
-1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2007, 2008, 2009, 2010
-Free Software Foundation, Inc.
-EOF
-compare - $TMP.2 <<EOF || exit 1
-# Copyright (C) 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998,
-# 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2007, 2008, 2009, 2010 Free
-# Software Foundation, Inc.
-EOF
-compare - $TMP.3 <<EOF || exit 1
-/*
- * Copyright &copy; 1990, 2005, 2007, 2008, 2009, 2010 Free Software
- * Foundation, Inc.
- */
-EOF
-compare - $TMP.4 <<EOF || exit 1
-## Copyright (C) 1990-2005, 2007-2009 Free Software
-#  Foundation, Inc.
-EOF
-compare - $TMP.5 <<EOF || exit 1
-Copyright (C) 1990-2005, 2007-2009 Acme, Inc.
-EOF
-compare - $TMP.6 <<EOF || exit 1
-## Copyright (C) 1990-2005, 2007-2009 Free Software
-#  Foundation, Inc.
-
-Copyright (C) 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998,
-1999, 2000, 2001, 2002, 2003, 2004, 2005, 2007, 2008, 2009, 2010 Free
-Software Foundation, Inc.
-EOF
-compare - $TMP.7 <<EOF || exit 1
-Copyright (C) 1990-2005, 2007-2009 Acme, Inc.
-
-# Copyright (C) 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998,
-# 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2007, 2008, 2009, 2010 Free
-# Software Foundation, Inc.
+Copy$ight \(co 2009\(en2010 Free Software Foundation, Inc.
 EOF
 
 rm $TMP*
@@ -237,7 +322,7 @@ rm $TMP*
 TMP=$TMP_BASE-current-year
 YEAR=`date +%Y`
 cat > $TMP <<EOF
-'\" Copyright (C) 2006
+'\" Copy$ight (C) 2006
 '\" Free Software Foundation,
 '\" Inc.
 EOF
@@ -252,7 +337,7 @@ UPDATE_COPYRIGHT_USE_INTERVALS=1 \
 compare /dev/null $TMP-stdout || exit 1
 compare /dev/null $TMP-stderr || exit 1
 compare - $TMP <<EOF || exit 1
-'\" Copyright (C) 2006, $YEAR Free Software Foundation, Inc.
+'\" Copy$ight (C) 2006, $YEAR Free Software Foundation, Inc.
 EOF
 rm $TMP*
 
@@ -264,7 +349,7 @@ TMP=$TMP_BASE-surrounding-text
 cat > $TMP <<EOF
     Undisturbed text.
 dnl Undisturbed text.
-dnl Copyright (C) 89
+dnl Copy$ight (C) 89
 dnl Free Software Foundation, Inc.
 dnl   Undisturbed text.
 EOF
@@ -275,7 +360,7 @@ compare /dev/null $TMP-stderr || exit 1
 compare - $TMP <<EOF || exit 1
     Undisturbed text.
 dnl Undisturbed text.
-dnl Copyright (C) 1989, 2010 Free Software Foundation, Inc.
+dnl Copy$ight (C) 1989, 2010 Free Software Foundation, Inc.
 dnl   Undisturbed text.
 EOF
 rm $TMP*
@@ -286,7 +371,7 @@ rm $TMP*
 
 TMP=$TMP_BASE-widest-prefix
 cat > $TMP <<EOF
-#### Copyright (C) 1976, 1977, 1978, 1979, 1980, 1981, 1982, 1983, 1984, 1985,
+#### Copy$ight (C) 1976, 1977, 1978, 1979, 1980, 1981, 1982, 1983, 1984, 1985,
 #### 1986, 1987, 1988, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007,
 #### 2008 Free Software Foundation, Inc.
 EOF
@@ -295,7 +380,7 @@ UPDATE_COPYRIGHT_YEAR=2010 \
 compare /dev/null $TMP-stdout || exit 1
 compare /dev/null $TMP-stderr || exit 1
 compare - $TMP <<EOF || exit 1
-#### Copyright (C) 1976, 1977, 1978, 1979, 1980, 1981, 1982, 1983, 1984,
+#### Copy$ight (C) 1976, 1977, 1978, 1979, 1980, 1981, 1982, 1983, 1984,
 #### 1985, 1986, 1987, 1988, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
 #### 2006, 2007, 2008, 2010 Free Software Foundation, Inc.
 EOF
@@ -304,7 +389,7 @@ UPDATE_COPYRIGHT_YEAR=2011 UPDATE_COPYRIGHT_USE_INTERVALS=1 \
 compare /dev/null $TMP-stdout || exit 1
 compare /dev/null $TMP-stderr || exit 1
 compare - $TMP <<EOF || exit 1
-#### Copyright (C) 1976-1988, 1999-2008, 2010-2011 Free Software
+#### Copy$ight (C) 1976-1988, 1999-2008, 2010-2011 Free Software
 #### Foundation, Inc.
 EOF
 rm $TMP*
@@ -326,7 +411,7 @@ compare - $TMP-stderr <<EOF || exit 1
 $TMP: warning: copyright statement not found
 EOF
 compare - $TMP <<EOF || exit 1
-####  Copyright (C) 1976, 1977, 1978, 1979, 1980, 1981, 1982, 1983, 1984, 1985,
+####  Copy$ight (C) 1976, 1977, 1978, 1979, 1980, 1981, 1982, 1983, 1984, 1985,
 ####  1986, 1987, 1988, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007,
 ####  2008 Free Software Foundation, Inc.
 EOF
@@ -338,12 +423,12 @@ rm $TMP*
 
 TMP=$TMP_BASE-blank-lines
 cat > $TMP <<EOF
-#Copyright (C) 1976, 1977, 1978, 1979, 1980, 1981, 1982, 1983, 1984, 1985,
+#Copy$ight (C) 1976, 1977, 1978, 1979, 1980, 1981, 1982, 1983, 1984, 1985,
 #
 #1986, 1987, 1988, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007,
 #2008 Free Software Foundation, Inc.
 
-Copyright (C) 1976, 1977, 1978, 1979, 1980, 1981, 1982, 1983, 1984, 1985,
+Copy$ight (C) 1976, 1977, 1978, 1979, 1980, 1981, 1982, 1983, 1984, 1985,
 
 1986, 1987, 1988, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007,
 2008 Free Software Foundation, Inc.
@@ -355,12 +440,12 @@ compare - $TMP-stderr <<EOF || exit 1
 $TMP: warning: copyright statement not found
 EOF
 compare - $TMP <<EOF || exit 1
-#Copyright (C) 1976, 1977, 1978, 1979, 1980, 1981, 1982, 1983, 1984, 1985,
+#Copy$ight (C) 1976, 1977, 1978, 1979, 1980, 1981, 1982, 1983, 1984, 1985,
 #
 #1986, 1987, 1988, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007,
 #2008 Free Software Foundation, Inc.
 
-Copyright (C) 1976, 1977, 1978, 1979, 1980, 1981, 1982, 1983, 1984, 1985,
+Copy$ight (C) 1976, 1977, 1978, 1979, 1980, 1981, 1982, 1983, 1984, 1985,
 
 1986, 1987, 1988, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007,
 2008 Free Software Foundation, Inc.
@@ -373,7 +458,7 @@ rm $TMP*
 
 TMP=$TMP_BASE-leading-tabs
 cat > $TMP <<EOF
-	Copyright (C) 87, 88, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 98,
+	Copy$ight (C) 87, 88, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 98,
 	 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Free
 	Software Foundation, Inc.
 EOF
@@ -382,7 +467,7 @@ UPDATE_COPYRIGHT_YEAR=2010 \
 compare /dev/null $TMP-stdout || exit 1
 compare /dev/null $TMP-stderr || exit 1
 compare - $TMP <<EOF || exit 1
-	Copyright (C) 1987, 1988, 1991, 1992, 1993, 1994, 1995, 1996,
+	Copy$ight (C) 1987, 1988, 1991, 1992, 1993, 1994, 1995, 1996,
 	1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
 	2007, 2008, 2009, 2010 Free Software Foundation, Inc.
 EOF
@@ -391,7 +476,7 @@ UPDATE_COPYRIGHT_YEAR=2011 UPDATE_COPYRIGHT_USE_INTERVALS=1 \
 compare /dev/null $TMP-stdout || exit 1
 compare /dev/null $TMP-stderr || exit 1
 compare - $TMP <<EOF || exit 1
-	Copyright (C) 1987-1988, 1991-2011 Free Software Foundation,
+	Copy$ight (C) 1987-1988, 1991-2011 Free Software Foundation,
 	Inc.
 EOF
 rm $TMP*
@@ -402,7 +487,7 @@ rm $TMP*
 
 TMP=$TMP_BASE-unusual-ws
 cat > $TMP <<EOF
-		# Copyright (C) 87-88, 1991, 1992, 1993, 1994, 1995, 1996, 1997,
+		# Copy$ight (C) 87-88, 1991, 1992, 1993, 1994, 1995, 1996, 1997,
 		# 98, 1999, 2000, 2001, 2002, 2003,     		  2004, 2005, 2006, 2007, 2008,
 		# 2009 Free Software Foundation, Inc.
 EOF
@@ -411,7 +496,7 @@ UPDATE_COPYRIGHT_YEAR=2010 \
 compare /dev/null $TMP-stdout || exit 1
 compare /dev/null $TMP-stderr || exit 1
 compare - $TMP <<EOF || exit 1
-		# Copyright (C) 1987, 1988, 1991, 1992, 1993, 1994,
+		# Copy$ight (C) 1987, 1988, 1991, 1992, 1993, 1994,
 		# 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
 		# 2004, 2005, 2006, 2007, 2008, 2009, 2010 Free Software
 		# Foundation, Inc.
@@ -421,7 +506,7 @@ UPDATE_COPYRIGHT_YEAR=2011 UPDATE_COPYRIGHT_USE_INTERVALS=1 \
 compare /dev/null $TMP-stdout || exit 1
 compare /dev/null $TMP-stderr || exit 1
 compare - $TMP <<EOF || exit 1
-		# Copyright (C) 1987-1988, 1991-2011 Free Software
+		# Copy$ight (C) 1987-1988, 1991-2011 Free Software
 		# Foundation, Inc.
 EOF
 UPDATE_COPYRIGHT_YEAR=2011 UPDATE_COPYRIGHT_USE_INTERVALS=2 \
@@ -429,7 +514,7 @@ UPDATE_COPYRIGHT_YEAR=2011 UPDATE_COPYRIGHT_USE_INTERVALS=2 \
 compare /dev/null $TMP-stdout || exit 1
 compare /dev/null $TMP-stderr || exit 1
 compare - $TMP <<EOF || exit 1
-		# Copyright (C) 1987-2011 Free Software Foundation, Inc.
+		# Copy$ight (C) 1987-2011 Free Software Foundation, Inc.
 EOF
 rm $TMP*
 
@@ -438,8 +523,8 @@ rm $TMP*
 ## --------- ##
 
 TMP=$TMP_BASE-dos-eol
-tr @ '\015' > $TMP <<\EOF
-Rem Copyright (C) 87, 88, 1991, 1992, 1993, 1994, 1995, 1996, 1997,@
+tr @ '\015' > $TMP <<EOF
+Rem Copy$ight (C) 87, 88, 1991, 1992, 1993, 1994, 1995, 1996, 1997,@
 Rem 98, 1999, 2000, 2001, 2002, 2003,  2004, 2005, 2006, 2007, 2008,@
 Rem 2009 Free Software Foundation, Inc.@
 EOF
@@ -447,8 +532,8 @@ UPDATE_COPYRIGHT_YEAR=2010 \
   update-copyright $TMP 1> $TMP-stdout 2> $TMP-stderr
 compare /dev/null $TMP-stdout || exit 1
 compare /dev/null $TMP-stderr || exit 1
-tr @ '\015' > $TMP-exp <<\EOF
-Rem Copyright (C) 1987, 1988, 1991, 1992, 1993, 1994, 1995, 1996, 1997,@
+tr @ '\015' > $TMP-exp <<EOF
+Rem Copy$ight (C) 1987, 1988, 1991, 1992, 1993, 1994, 1995, 1996, 1997,@
 Rem 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,@
 Rem 2009, 2010 Free Software Foundation, Inc.@
 EOF
@@ -461,7 +546,7 @@ rm $TMP*
 
 TMP=$TMP_BASE-omitted-circle-c
 cat > $TMP <<EOF
-  Copyright 87, 88, 1991, 1992, 1993, 1994, 1995, 1996, 1997,
+  Copy$ight 87, 88, 1991, 1992, 1993, 1994, 1995, 1996, 1997,
   98, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
   2009 Free Software Foundation, Inc.
 EOF
@@ -470,7 +555,7 @@ UPDATE_COPYRIGHT_YEAR=2010 \
 compare /dev/null $TMP-stdout || exit 1
 compare /dev/null $TMP-stderr || exit 1
 compare - $TMP <<EOF || exit 1
-  Copyright 1987, 1988, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998,
+  Copy$ight 1987, 1988, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998,
   1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
   Free Software Foundation, Inc.
 EOF
@@ -482,65 +567,65 @@ rm $TMP*
 
 TMP=$TMP_BASE-c-style-comments
 cat > $TMP.star <<EOF
-/*  Copyright 87, 88, 1991, 1992, 1993, 1994, 1995, 1996, 1997,
+/*  Copy$ight 87, 88, 1991, 1992, 1993, 1994, 1995, 1996, 1997,
  *  98, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
  *  2009 Free Software Foundation, Inc.  */
 EOF
 cat > $TMP.space <<EOF
-  /*Copyright 87, 88, 1991, 1992, 1993, 1994, 1995, 1996, 1997,
+  /*Copy$ight 87, 88, 1991, 1992, 1993, 1994, 1995, 1996, 1997,
     98, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
     2009 Free Software Foundation, Inc.  */
 EOF
 cat > $TMP.single-line <<EOF
-/*   Copyright 87, 1991, 1992 Free Software Foundation, Inc.  */
+/*   Copy$ight 87, 1991, 1992 Free Software Foundation, Inc.  */
 EOF
 cat > $TMP.single-line-wrapped <<EOF
- /* Copyright 1988, 1991, 1992, 1993 Free Software Foundation, Inc.  */
+ /* Copy$ight 1988, 1991, 1992, 1993 Free Software Foundation, Inc.  */
 EOF
 cat > $TMP.extra-text-star <<EOF
- /* Copyright 1987, 1988, 1991, 1992 Free Software Foundation, Inc.  End
+ /* Copy$ight 1987, 1988, 1991, 1992 Free Software Foundation, Inc.  End
   * More comments.  */
 EOF
 cat > $TMP.extra-text-space <<EOF
- /* Copyright 1987, 1988, 1991, 1992 Free Software Foundation, Inc. ***
+ /* Copy$ight 1987, 1988, 1991, 1992 Free Software Foundation, Inc. ***
     * End of comments. */
 EOF
 cat > $TMP.two-digit-final-is-substr-of-first <<EOF
- /* Copyright 1991, 99 Free Software Foundation, Inc. */
+ /* Copy$ight 1991, 99 Free Software Foundation, Inc. */
 EOF
 UPDATE_COPYRIGHT_YEAR=2010 \
   update-copyright $TMP.* 1> $TMP-stdout 2> $TMP-stderr
 compare /dev/null $TMP-stdout || exit 1
 compare /dev/null $TMP-stderr || exit 1
 compare - $TMP.star <<EOF || exit 1
-/*  Copyright 1987, 1988, 1991, 1992, 1993, 1994, 1995, 1996, 1997,
+/*  Copy$ight 1987, 1988, 1991, 1992, 1993, 1994, 1995, 1996, 1997,
  *  1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
  *  2009, 2010 Free Software Foundation, Inc.  */
 EOF
 compare - $TMP.space <<EOF || exit 1
-  /*Copyright 1987, 1988, 1991, 1992, 1993, 1994, 1995, 1996, 1997,
+  /*Copy$ight 1987, 1988, 1991, 1992, 1993, 1994, 1995, 1996, 1997,
     1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
     2009, 2010 Free Software Foundation, Inc.  */
 EOF
 compare - $TMP.single-line <<EOF || exit 1
-/*   Copyright 1987, 1991, 1992, 2010 Free Software Foundation, Inc.  */
+/*   Copy$ight 1987, 1991, 1992, 2010 Free Software Foundation, Inc.  */
 EOF
 compare - $TMP.single-line-wrapped <<EOF || exit 1
- /* Copyright 1988, 1991, 1992, 1993, 2010 Free Software Foundation,
+ /* Copy$ight 1988, 1991, 1992, 1993, 2010 Free Software Foundation,
   * Inc.  */
 EOF
 compare - $TMP.extra-text-star <<EOF || exit 1
- /* Copyright 1987, 1988, 1991, 1992, 2010 Free Software Foundation,
+ /* Copy$ight 1987, 1988, 1991, 1992, 2010 Free Software Foundation,
   * Inc.  End
   * More comments.  */
 EOF
 compare - $TMP.extra-text-space <<EOF || exit 1
- /* Copyright 1987, 1988, 1991, 1992, 2010 Free Software Foundation,
+ /* Copy$ight 1987, 1988, 1991, 1992, 2010 Free Software Foundation,
     Inc. ***
     * End of comments. */
 EOF
 compare - $TMP.two-digit-final-is-substr-of-first <<EOF || exit 1
- /* Copyright 1991, 1999, 2010 Free Software Foundation, Inc. */
+ /* Copy$ight 1991, 1999, 2010 Free Software Foundation, Inc. */
 EOF
 rm $TMP*
 
