@@ -1,5 +1,5 @@
 /* frcode -- front-compress a sorted list
-   Copyright (C) 1994-2022 Free Software Foundation, Inc.
+   Copyright (C) 1994-2024 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -78,14 +78,12 @@
 
 /* gnulib headers. */
 #include "closeout.h"
-#include "error.h"
 #include "progname.h"
 #include "xalloc.h"
 
 /* find headers. */
 #include "system.h"
 #include "bugreports.h"
-#include "die.h"
 #include "findutils-version.h"
 #include "gcc-function-attributes.h"
 #include "locatedb.h"
@@ -165,24 +163,24 @@ get_seclevel (char *s)
   result = strtol (s, &p, 10);
   if ((0==result) && (p == optarg))
     {
-      die (EXIT_FAILURE, 0,
-	   _("You need to specify a security level as a decimal integer."));
+      error (EXIT_FAILURE, 0,
+	     _("You need to specify a security level as a decimal integer."));
       /*NOTREACHED*/
       return -1;
     }
   else if ((LONG_MIN==result || LONG_MAX==result) && errno)
 
     {
-      die (EXIT_FAILURE, 0,
-	   _("Security level %s is outside the convertible range."), s);
+      error (EXIT_FAILURE, 0,
+	     _("Security level %s is outside the convertible range."), s);
       /*NOTREACHED*/
       return -1;
     }
   else if (*p)
     {
       /* Some suffix exists */
-      die (EXIT_FAILURE, 0,
-	   _("Security level %s has unexpected suffix %s."), s, p);
+      error (EXIT_FAILURE, 0,
+	     _("Security level %s has unexpected suffix %s."), s, p);
       /*NOTREACHED*/
       return -1;
     }
@@ -196,7 +194,7 @@ static void
 outerr (void)
 {
   /* Issue the same error message as closeout () would. */
-  die (EXIT_FAILURE, errno, _("write error"));
+  error (EXIT_FAILURE, errno, _("write error"));
 }
 
 int
@@ -219,7 +217,7 @@ main (int argc, char **argv)
 
   if (atexit (close_stdout))
     {
-      die (EXIT_FAILURE, errno, _("The atexit library function failed"));
+      error (EXIT_FAILURE, errno, _("The atexit library function failed"));
     }
 
   pathsize = oldpathsize = 1026; /* Increased as necessary by getline.  */
@@ -242,9 +240,9 @@ main (int argc, char **argv)
 	slocate_seclevel = get_seclevel (optarg);
 	if (slocate_seclevel < 0 || slocate_seclevel > 1)
 	  {
-	    die (EXIT_FAILURE, 0,
-		 _("slocate security level %ld is unsupported."),
-		 slocate_seclevel);
+	    error (EXIT_FAILURE, 0,
+		   _("slocate security level %ld is unsupported."),
+		   slocate_seclevel);
 	  }
 	break;
 
@@ -279,7 +277,7 @@ main (int argc, char **argv)
       if (fwrite (LOCATEDB_MAGIC, 1, sizeof (LOCATEDB_MAGIC), stdout)
 	  != sizeof (LOCATEDB_MAGIC))
 	{
-	  die (EXIT_FAILURE, errno, _("Failed to write to standard output"));
+	  error (EXIT_FAILURE, errno, _("Failed to write to standard output"));
 	}
     }
 
